<?php

/*
Class Name: VI_WOOCOMMERCE_COUPON_BOXP_Admin_Active_Campaign
Author: Andy Ha (support@villatheme.com)
Author URI: http://villatheme.com
Copyright 2015 villatheme.com. All rights reserved.
*/
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

class VI_WOOCOMMERCE_COUPON_BOXP_Admin_Active_Campaign {
    
    protected $settings;
    protected $api_key;
    protected $url;
    
    function __construct() {
        $this->settings = new VI_WOOCOMMERCE_COUPON_BOX_DATA();
        $this->api_key  = $this->settings->get_params( 'wcb_active_campaign_api' );
        $this->url      = $this->settings->get_params( 'wcb_active_campaign_url' );
    }
    
    public function contact_add( $email, $list = '', $firstname = '', $lastname = '', $phone = '', $orgname = '', $tags = 'api' ) {
        if ( ! $this->api_key || ! $this->url ) {
            return '';
        }
        
        $url = rtrim( $this->url, '/ ' ) . '/admin/api.php';
        
        $params = array(
            'api_key'    => $this->api_key,
            'api_action' => 'contact_add',
            'api_output' => 'serialize',
        );
        
        $post = array(
            'email'      => $email,
            'first_name' => $firstname,
            'last_name'  => $lastname,
            'phone'      => $phone,
            'orgname'    => $orgname,
            'tags'       => $tags,
        );
        
        if ( $list ) {
            $post[ 'p[' . $list . ']' ]      = $list;
            $post[ 'status[' . $list . ']' ] = 1;
        }
        
        $query = http_build_query( $params );
        $api_url = $url . '?' . $query;
        
        $args = array(
            'body'        => $post,
            'headers'     => array(
                'Content-Type' => 'application/x-www-form-urlencoded',
            ),
            'timeout'     => 10,
            'sslverify'   => false,
        );
        
        wp_remote_post( $api_url, $args );
    }

	function contact_unsubscribe( $email, $list_id ) {
		$api_url = rtrim( $this->url, '/ ' );
		$search_url = esc_url( $api_url ) . '/api/3/contacts?email=' . urlencode( $email );

		// Step 1: Find the Contact ID using Email
		$search_response = wp_remote_get( $search_url, [
			'headers' => [
				'Api-Token' => $this->api_key,
				'Content-Type' => 'application/json',
			],
		]);

		if ( is_wp_error( $search_response ) ) {
			return 'Error: ' . $search_response->get_error_message();
		}

		$search_body = json_decode( wp_remote_retrieve_body( $search_response ), true );

		if ( ! is_array( $search_body ) || ! isset( $search_body['contacts'] ) || empty( $search_body['contacts'] ) ) {
			return "Error: Contact not found.";
		}

		$contact_id = '';
		if ( is_array( $search_body['contacts'] ) && isset( $search_body['contacts'][0] ) &&
		     is_array( $search_body['contacts'] ) && isset( $search_body['contacts'][0]['id'] ) ) {
			$contact_id = $search_body['contacts'][0]['id'];
		}
		if ( ! $contact_id ) {
			return "Error: Contact not found.";
		}

		// Step 2: Unsubscribe Contact from the List
		$delete_url = esc_url( $api_url ) . '/api/3/contactLists';

		$delete_data = [
			'contactList' => [
				'list' => $list_id,
				'contact' => $contact_id,
				'status' => 2 // 2 = Unsubscribe
			]
		];

		$delete_response = wp_remote_post( $delete_url, [
			'method'    => 'POST',
			'headers'   => [
				'Api-Token'    => $this->api_key,
				'Content-Type' => 'application/json',
			],
			'body'      => json_encode($delete_data),
		] );

		if ( is_wp_error( $delete_response ) ) {
			return 'Error: ' . $delete_response->get_error_message();
		}

		$http_code = wp_remote_retrieve_response_code( $delete_response );
		$delete_body = json_decode( wp_remote_retrieve_body( $delete_response ), true );

		if ($http_code == 201) {
			return "Member successfully unsubscribed from the list.";
		} else {
			return "Error: " . json_encode($delete_body);
		}
	}
    
    public function list_list( $name ) {
        if ( ! $this->api_key || ! $this->url ) {
            return '';
        }
        
        $url = rtrim( $this->url, '/ ' ) . '/admin/api.php';
        
        $params = array(
            'api_key'      => $this->api_key,
            'api_action'   => 'list_list',
            'api_output'   => 'json',
            'filters[name]' => $name,
        );
        
        $query = http_build_query( $params );
        
        $api_url = $url . '?' . $query;
        
        $args = array(
            'headers' => array(
                'Content-Type' => 'application/json',
            ),
            'timeout' => 10,
            'sslverify' => false,
        );
        
        $response = wp_remote_get( $api_url, $args );
        
        if ( is_wp_error( $response ) ) {
            return '';
        }
        
        $response_body = wp_remote_retrieve_body( $response );
        
        if ( ! $response_body ) {
            return '';
        }
        
        $result = json_decode( $response_body, true );
        
        return $result;
        
    }
    
    public function list_view( $id ) {
        if ( ! $this->api_key || ! $this->url ) {
            return '';
        }
        
        $url    = $this->url;
        $params = array(
            'api_key'    => $this->api_key,
            'api_action' => 'list_view',
            'api_output' => 'serialize',
            'id'         => $id,
        );
        
        // This section takes the input fields and converts them to the proper format
        $query = http_build_query( $params );
        
        // clean up the url
        $url = rtrim( $url, '/ ' ) . '/admin/api.php?' . $query;
        
        $args = array(
            'headers' => array(
                'Content-Type' => 'application/json',
            ),
            'timeout' => 10,
            'sslverify' => false,
        );
        
        $response = wp_remote_get( $url, $args );
        
        if ( is_wp_error( $response ) ) {
            return '';
        }
        
        $response_body = wp_remote_retrieve_body( $response );
        
        if ( ! $response_body ) {
            return '';
        }
        
        // This line takes the response and breaks it into an array using:
        // JSON decoder
        //$result = json_decode($response);
        // unserializer
        $result = unserialize( $response_body );
        
        return $result;
        
    }
    
}
