<?php
/*
Class Name: VI_WOOCOMMERCE_COUPON_BOX_Admin_Sendgrid
Author: Andy Ha (support@villatheme.com)
Author URI: http://villatheme.com
Copyright 2015 villatheme.com. All rights reserved.
*/
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class VI_WOOCOMMERCE_COUPON_BOX_Admin_Klaviyo {
	protected $settings;
	protected $api_key;

	function __construct() {
		$this->settings = new VI_WOOCOMMERCE_COUPON_BOX_DATA();
		$this->api_key  = $this->settings->get_params( 'wcb_klaviyo_api' );
	}

	public function get_lists() {
		if ( ! $this->api_key ) {
			return array();
		}
		$result = [];
		try {

			$headers = [
				'Authorization' => 'Klaviyo-API-Key ' . $this->api_key,
				'Accept'        => 'application/json',
				'revision'      => '2023-02-22'
			];

			$r         = wp_remote_get( 'https://a.klaviyo.com/api/lists/', [
				'headers' => $headers
			] );
			$body      = json_decode( wp_remote_retrieve_body( $r ), true );

			if ( isset( $body['data'] ) ) {
				$body_data = $body['data'];

				foreach ( $body_data as $list_data ) {
					$list_attribute = $list_data['attributes'] ?? [];
					$result[]       = [
						'list_id'   => $list_data['id'] ?? '',
						'list_name' => $list_attribute['name'] ?? '',
					];
				}
			}

//			$r = wp_remote_get( 'https://a.klaviyo.com/api/v2/lists?api_key=' . $this->api_key );
//
//			$body = wp_remote_retrieve_body( $r );

			return $result;

		} catch ( \Exception $e ) {

		}

		return $result;
	}

	public function add_recipient( $email = '', $list_id = '', $firstname = '', $lastname = '', $phone = '' ) {
		if ( ! $this->api_key || ! $email || ! $list_id ) {
			return;
		}
		$add_profile = $this->add_profile( $email, $firstname, $lastname, $phone );
		if ( ! empty( $add_profile ) ) {
			$data_profile = $add_profile['data'] ?? [];
			$id_profile   = $data_profile['id'] ?? '';
			if ( ! empty( $id_profile ) ) {
				$headers = [
					'Authorization' => 'Klaviyo-API-Key ' . $this->api_key,
					'Accept'        => 'application/json',
					'content-type'  => 'application/json',
					'revision'      => '2023-02-22'
				];
				$body    = wp_json_encode( [
					'data' => [
						[
							'type' => 'profile',
							'id'   => $id_profile,/*ID profile*/
						]
					]

				] );
				try {
					$r = wp_remote_post( "https://a.klaviyo.com/api/lists/{$list_id}/relationships/profiles/", [
						'headers' => $headers,
						'body'    => $body,
					] );
				} catch ( Exception $e ) {

				}

			}
		}
	}

	public function add_profile( $email = '', $firstname = '', $lastname = '', $phone = '' ) {
		if ( ! empty( $firstname ) && empty( $lastname ) ) {
			$arr_name = explode( ' ', $firstname );/*separates the full name if there are spaces, Ex: Join Smith*/
			$lastname = $arr_name[1] ?? ''; /*get the last name: Ex: Smith*/
		}
		$attributes_profile = [
			'email'      => $email,
			'first_name' => $firstname,
			'last_name'  => $lastname
		];
		if ( ! empty( $phone ) ) {
			$attributes_profile['phone_number'] = strval( $phone );
		}
		$body = wp_json_encode( [
			'data' => [
				'type'       => 'profile',
				'attributes' => $attributes_profile
			]
		] );

		$headers = [
			'Authorization' => 'Klaviyo-API-Key ' . $this->api_key,
			'Accept'        => 'application/json',
			'content-type'  => 'application/json',
			'revision'      => '2023-02-22'
		];

		try {
			$r    = wp_remote_post( "https://a.klaviyo.com/api/profiles/", [
				'headers'     => $headers,
				'body'        => $body,
				'method'      => 'POST',
				'data_format' => 'body'
			] );
			$body = json_decode( wp_remote_retrieve_body( $r ), true );
			if ( ! empty( $body['errors'] ) ) {
				$profile_meta = $body['errors'][0]['meta'];
				$profile_id_exist = $profile_meta['duplicate_profile_id']??'';
				$r_2 = wp_remote_get( "https://a.klaviyo.com/api/profiles/".$profile_id_exist, [
					'headers'     => $headers,
				] );
				$body = json_decode( wp_remote_retrieve_body( $r_2 ), true );
			}

			return ( $body );

		} catch ( \Exception $e ) {
		}

		return [];

	}

}
