<?php
/*
Class Name: VI_WOOCOMMERCE_COUPON_BOX_Admin_Mailchimp
Author: Andy Ha (support@villatheme.com)
Author URI: http://villatheme.com
Copyright 2015 villatheme.com. All rights reserved.
*/
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class VI_WOOCOMMERCE_COUPON_BOX_Admin_Mailchimp {
	protected $settings;
	protected $api_key;

	function __construct() {
		$this->settings = new VI_WOOCOMMERCE_COUPON_BOX_DATA();
		$this->api_key  = $this->settings->get_params( 'wcb_api' );
	}

	function get_list( $list_id ) {
		if ( ! $this->api_key || ! $list_id ) {
			return '';
		}

		$dataCenter = substr( $this->api_key, strpos( $this->api_key, '-' ) + 1 );
		$url        = 'https://' . $dataCenter . '.api.mailchimp.com/3.0/lists/' . $list_id;

		$args = array(
			'headers'   => array(
				'Authorization' => 'Basic ' . base64_encode( 'user:' . $this->api_key ),
				'Content-Type'  => 'application/json',
			),
			'timeout'   => 10,
			'sslverify' => false,
		);

		$response = wp_remote_get( $url, $args );

		if ( is_wp_error( $response ) ) {
			return '';
		}

		$response_body = wp_remote_retrieve_body( $response );
		$data          = json_decode( $response_body );

		return $data;
	}

	function get_lists() {
		if ( $this->api_key ) {
			$dash_position = strpos( $this->api_key, '-' );
			if ( $dash_position !== false ) {
				$url  = 'https://' . substr( $this->api_key, $dash_position + 1 ) . '.api.mailchimp.com/3.0/lists?fields=lists.name,lists.id&count=1000';
				$auth = base64_encode( 'user:' . esc_attr( $this->api_key ) );

				try {
					$r = wp_remote_get( $url, [
						'headers' => [
							'Authorization' => "Basic $auth",
							'Accept'        => 'application/json',
							'Content-Type'  => 'application/json',
						],
					] );

					$body       = wp_remote_retrieve_body( $r );
					$lists_data = json_decode( $body );

					if ( property_exists( $lists_data, 'lists' ) ) {
						$data  = $lists_data->lists;
						$lists = array();
						if ( count( $data ) ) {

							foreach ( $data as $list ) {
								$lists[ $list->id ] = $list->name;
							}

						} else {
							return false;
						}

						return $lists;
					} else {
						return false;
					}

				} catch ( \Exception $e ) {

					return false;
				}
			} else {
				return false;
			}
		} else {
			return false;
		}
	}

	function add_email( $email, $list_id, $merge_fields ) {

		if ( $this->api_key && $list_id ) {

			$merge_fields = wp_parse_args( $merge_fields, array(
				'FNAME'    => '',
				'LNAME'    => '',
				'PHONE'    => '',
				'BIRTHDAY' => ''
			) );

			$data = array(
				'email_address' => $email,
				'status'        => $this->settings->get_params( 'wcb_mailchimp_double_optin' ) ? 'pending' : 'subscribed',
				'merge_fields'  => $merge_fields
			);

			$dataCenter = substr( $this->api_key, strpos( $this->api_key, '-' ) + 1 );
			$url        = 'https://' . esc_attr( $dataCenter ) . '.api.mailchimp.com/3.0/lists/' . esc_attr( $list_id ) . '/members/' . md5( strtolower( $email ) );
			$auth       = base64_encode( 'user:' . esc_attr( $this->api_key ) );

			try {
				$r = wp_remote_post( $url, [
					'method'  => 'PUT',
					'headers' => [
						'Authorization' => "Basic $auth",
						'Accept'        => 'application/json',
						'Content-Type'  => 'application/json',
					],
					'body'    => wp_json_encode( $data ),
				] );

				$body = wp_remote_retrieve_body( $r );
				$data = json_decode( $body );
				switch ( $data->status ) {
					case 'subscribed':
						return true;
						break;
					default:
						return false;
				}

			} catch ( \Exception $e ) {

				return false;
			}

		} else {
			return false;
		}
	}

	function unsubscribe_email( $email, $list_id ) {

		if ( $this->api_key && $list_id ) {

			$dataCenter = substr( $this->api_key, strpos( $this->api_key, '-' ) + 1 );
			$url        = 'https://' . esc_attr( $dataCenter ) . '.api.mailchimp.com/3.0/lists/' . esc_attr( $list_id ) . '/members/' . md5( strtolower( $email ) );

			try {
				$response = wp_remote_request( $url, [
					'method'  => 'DELETE',
					'headers' => [
						'Authorization' => 'Basic ' . base64_encode( 'user:' . $this->api_key ),
						'Content-Type'  => 'application/json'
					],
				] );
				if ( is_wp_error( $response ) ) {
					return 'Error: ' . $response->get_error_message();
				}

				$http_code = wp_remote_retrieve_response_code( $response );

				if ( $http_code == 204 ) {
					return "Member successfully deleted.";
				} elseif ( $http_code == 404 ) {
					return "Member not found.";
				} else {
					return "Error: " . wp_remote_retrieve_body( $response );
				}

			} catch ( \Exception $e ) {

				return false;
			}

		} else {
			return false;
		}
	}
}
