<?php

/*
Class Name: VI_WOOCOMMERCE_COUPON_BOX_Admin_Sendgrid
Author: Andy Ha (support@villatheme.com)
Author URI: http://villatheme.com
Copyright 2015 villatheme.com. All rights reserved.
*/
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

class VI_WOOCOMMERCE_COUPON_BOX_Admin_Sendgrid {
    
    protected $settings;
    protected $api_key;
    
    function __construct() {
        $this->settings = new VI_WOOCOMMERCE_COUPON_BOX_DATA();
        $this->api_key  = $this->settings->get_params( 'wcb_sendgrid_api' );
    }
    
    public function get_custom_field() {
        if ( ! $this->api_key ) {
            return;
        }
        
        $url = "https://api.sendgrid.com/v3/contactdb/custom_fields";
        
        $args = array(
            'headers'     => array(
                'Authorization' => 'Bearer ' . $this->api_key,
                'Content-Type'  => 'application/json',
            ),
            'timeout'     => 30,
            'redirection' => 10,
        );
        
        $response = wp_remote_get( $url, $args );
        
        if ( is_wp_error( $response ) ) {
            $error_message = $response->get_error_message();
            echo wp_kses_post( "Request Error: " . $error_message );
        } else {
            $response_body = wp_remote_retrieve_body( $response );
            $data          = json_decode( $response_body, true );
            print_r( $data );
        }
    }
    
    public function add_custom_field( $name = '', $type = 'text' ) {
        if ( ! $this->api_key ) {
            return;
        }
        if ( ! $name ) {
            return;
        }
        
        $post_field = array(
            'name' => $name,
            'type' => $type,
        );
        $post_field = wp_json_encode( $post_field );
        
        $url = "https://api.sendgrid.com/v3/contactdb/custom_fields";
        
        $args = array(
            'method'      => 'POST',
            'body'        => $post_field,
            'headers'     => array(
                'Authorization' => 'Bearer ' . $this->api_key,
                'Content-Type'  => 'application/json',
            ),
            'timeout'     => 30,
            'redirection' => 10,
        );
        
        $response = wp_remote_post( $url, $args );
        
        if ( is_wp_error( $response ) ) {
            $error_message = $response->get_error_message();
            echo wp_kses_post( "Request Error: " . $error_message );
        } else {
            $response_body = wp_remote_retrieve_body( $response );
            $data          = json_decode( $response_body, true );
            print_r( $data );
        }
    }
    
    public function get_list_id( $list = '' ) {
        if ( ! $this->api_key ) {
            return '';
        }
        if ( ! $list ) {
            return '';
        }
        
        $url = "https://api.sendgrid.com/v3/contactdb/lists";
        
        $args = array(
            'headers'     => array(
                'Authorization' => 'Bearer ' . $this->api_key,
                'Content-Type'  => 'application/json',
            ),
            'timeout'     => 30,
            'redirection' => 10,
        );
        
        $response = wp_remote_get( $url, $args );
        
        if ( is_wp_error( $response ) ) {
            return '';
        }
        
        $response_body = wp_remote_retrieve_body( $response );
        $data          = json_decode( $response_body );
        
        if ( ! empty( $data->lists ) && count( $data->lists ) ) {
            foreach ( $data->lists as $item ) {
                if ( isset( $item->name ) && $list == $item->name ) {
                    return isset( $item->id ) ? $item->id : '';
                }
            }
        }
        
        return '';
    }
    
    public function get_list( $list_id ) {
        if ( ! $this->api_key || ! $list_id ) {
            return array();
        }
        
        $url = "https://api.sendgrid.com/v3/contactdb/lists/" . $list_id;
        
        $args = array(
            'headers'     => array(
                'Authorization' => 'Bearer ' . $this->api_key,
                'Content-Type'  => 'application/json',
            ),
            'timeout'     => 30,
            'redirection' => 10,
        );
        
        $response = wp_remote_get( $url, $args );
        
        if ( is_wp_error( $response ) ) {
            return array();
        }
        
        $response_body = wp_remote_retrieve_body( $response );
        $data          = json_decode( $response_body, true );
        
        return $data ? $data : array();
    }
    public function get_lists() {
        if ( ! $this->api_key ) {
            return array();
        }
        
        $url = "https://api.sendgrid.com/v3/marketing/lists";
        
        $args = array(
            'headers'     => array(
                'Authorization' => 'Bearer ' . $this->api_key,
                'Content-Type'  => 'application/json',
            ),
            'timeout'     => 30,
            'redirection' => 10,
        );
        
        $response = wp_remote_get( $url, $args );
        
        if ( is_wp_error( $response ) ) {
            return array();
        }
        
        $response_body = wp_remote_retrieve_body( $response );
        $data          = json_decode( $response_body, true );
        
        if ( isset( $data['errors'] ) && ! empty( $data['errors'] ) ) {
            return array();
        }
        
        if ( ! empty( $data['result'] ) && count( $data['result'] ) ) {
            return $data['result'];
        }
        
        return array();
    }
    
    public function add_recipient( $email = '', $firstname = '', $lastname = '' ) {
        if ( ! $this->api_key ) {
            return;
        }
        if ( ! $email ) {
            return;
        }
        
        $post_field = array(
            array(
                'email'      => $email,
                'first_name' => $firstname,
                'last_name'  => $lastname,
            )
        );
        
        $url = "https://api.sendgrid.com/v3/contactdb/recipients";
        
        $args = array(
            'body'    => wp_json_encode( $post_field ),
            'headers' => array(
                'Authorization' => 'Bearer ' . $this->api_key,
                'Content-Type'  => 'application/json',
            ),
            'timeout' => 30,
            'redirection' => 10,
        );
        
        wp_remote_post( $url, $args );
    }
    public function add_recipient_to_list( $email = '', $list_id = '' ) {
        if ( ! $this->api_key ) {
            return;
        }
        if ( ! $email || ! $list_id ) {
            return;
        }
        
        $args = [
            'method'  => 'PUT',
            'headers' => [
                "Authorization" => "Bearer " . $this->api_key,
                "Content-type"  => "application/json",
            ],
            'body'    => wp_json_encode( [
                'list_ids' => [ $list_id ],
                'contacts' => [
                    [ 'email' => $email ],
                ],
            ] ),
        ];
        
        wp_remote_request( 'https://api.sendgrid.com/v3/marketing/contacts', $args );
    }
    
}
